'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const cpp = require("child-process-promise");
const events_1 = require("events");
const fs = require("fs");
const os = require("os");
const path = require("path");
const ts_deferred_1 = require("ts-deferred");
const typescript_string_operations_1 = require("typescript-string-operations");
const component = require("../../common/component");
const errors_1 = require("../../common/errors");
const experimentStartupInfo_1 = require("../../common/experimentStartupInfo");
const log_1 = require("../../common/log");
const observableTimer_1 = require("../../common/observableTimer");
const utils_1 = require("../../common/utils");
const trialConfigMetadataKey_1 = require("../common/trialConfigMetadataKey");
const gpuScheduler_1 = require("./gpuScheduler");
const remoteMachineData_1 = require("./remoteMachineData");
const gpuData_1 = require("../common/gpuData");
const sshClientUtility_1 = require("./sshClientUtility");
const util_1 = require("../common/util");
const remoteMachineJobRestServer_1 = require("./remoteMachineJobRestServer");
const containerJobData_1 = require("../common/containerJobData");
const utils_2 = require("../../common/utils");
let RemoteMachineTrainingService = class RemoteMachineTrainingService {
    constructor(timer) {
        this.MAX_TRIAL_NUMBER_PER_SSHCONNECTION = 5;
        this.stopping = false;
        this.isMultiPhase = false;
        this.versionCheck = true;
        this.remoteOS = 'linux';
        this.metricsEmitter = new events_1.EventEmitter();
        this.trialJobsMap = new Map();
        this.trialSSHClientMap = new Map();
        this.machineSSHClientMap = new Map();
        this.gpuScheduler = new gpuScheduler_1.GPUScheduler(this.machineSSHClientMap);
        this.jobQueue = [];
        this.expRootDir = utils_1.getExperimentRootDir();
        this.remoteExpRootDir = this.getRemoteExperimentRootDir();
        this.timer = timer;
        this.log = log_1.getLogger();
        this.trialSequenceId = -1;
        this.logCollection = 'none';
        this.log.info('Construct remote machine training service.');
    }
    async run() {
        const restServer = component.get(remoteMachineJobRestServer_1.RemoteMachineJobRestServer);
        await restServer.start();
        restServer.setEnableVersionCheck = this.versionCheck;
        this.log.info('Run remote machine training service.');
        while (!this.stopping) {
            while (this.jobQueue.length > 0) {
                this.updateGpuReservation();
                const trialJobId = this.jobQueue[0];
                const prepareResult = await this.prepareTrialJob(trialJobId);
                if (prepareResult) {
                    this.jobQueue.shift();
                }
                else {
                    break;
                }
            }
            if (restServer.getErrorMessage) {
                throw new Error(restServer.getErrorMessage);
                this.stopping = true;
            }
            await utils_1.delay(3000);
        }
        this.log.info('Remote machine training service exit.');
    }
    async allocateSSHClientForTrial(trial) {
        const deferred = new ts_deferred_1.Deferred();
        if (!trial.rmMeta) {
            throw new Error(`rmMeta not set in trial ${trial.id}`);
        }
        let sshClientManager = this.machineSSHClientMap.get(trial.rmMeta);
        if (!sshClientManager) {
            throw new Error(`remoteSSHClient not initialized`);
        }
        let sshClient = await sshClientManager.getAvailableSSHClient();
        this.trialSSHClientMap.set(trial.id, sshClient);
        deferred.resolve();
        return deferred.promise;
    }
    releaseTrialSSHClient(trial) {
        if (!trial.rmMeta) {
            throw new Error(`rmMeta not set in trial ${trial.id}`);
        }
        let sshClientManager = this.machineSSHClientMap.get(trial.rmMeta);
        if (!sshClientManager) {
            throw new Error(`sshClientManager not initialized`);
        }
        sshClientManager.releaseConnection(this.trialSSHClientMap.get(trial.id));
    }
    async listTrialJobs() {
        const jobs = [];
        const deferred = new ts_deferred_1.Deferred();
        for (const [key, value] of this.trialJobsMap) {
            if (value.form.jobType === 'TRIAL') {
                jobs.push(await this.getTrialJob(key));
            }
        }
        ;
        deferred.resolve(jobs);
        return deferred.promise;
    }
    async getTrialJob(trialJobId) {
        const trialJob = this.trialJobsMap.get(trialJobId);
        if (!trialJob) {
            throw new errors_1.NNIError(errors_1.NNIErrorNames.NOT_FOUND, `trial job id ${trialJobId} not found`);
        }
        if (trialJob.status === 'RUNNING' || trialJob.status === 'UNKNOWN') {
            if (trialJob.rmMeta === undefined) {
                throw new Error(`rmMeta not set for submitted job ${trialJobId}`);
            }
            const sshClient = this.trialSSHClientMap.get(trialJob.id);
            if (!sshClient) {
                throw new Error(`Invalid job id: ${trialJobId}, cannot find ssh client`);
            }
            return this.updateTrialJobStatus(trialJob, sshClient);
        }
        else {
            return trialJob;
        }
    }
    addTrialJobMetricListener(listener) {
        this.metricsEmitter.on('metric', listener);
    }
    removeTrialJobMetricListener(listener) {
        this.metricsEmitter.off('metric', listener);
    }
    async submitTrialJob(form) {
        if (!this.trialConfig) {
            throw new Error('trial config is not initialized');
        }
        if (form.jobType === 'HOST') {
            return this.runHostJob(form);
        }
        else if (form.jobType === 'TRIAL') {
            const trialJobId = utils_1.uniqueString(5);
            const trialWorkingFolder = path.join(this.remoteExpRootDir, 'trials', trialJobId);
            const trialJobDetail = new remoteMachineData_1.RemoteMachineTrialJobDetail(trialJobId, 'WAITING', Date.now(), trialWorkingFolder, form, this.generateSequenceId());
            this.jobQueue.push(trialJobId);
            this.trialJobsMap.set(trialJobId, trialJobDetail);
            return Promise.resolve(trialJobDetail);
        }
        else {
            return Promise.reject(new Error(`Job form not supported: ${JSON.stringify(form)}, jobType should be HOST or TRIAL.`));
        }
    }
    async updateTrialJob(trialJobId, form) {
        const trialJobDetail = this.trialJobsMap.get(trialJobId);
        if (trialJobDetail === undefined) {
            throw new Error(`updateTrialJob failed: ${trialJobId} not found`);
        }
        if (form.jobType === 'TRIAL') {
            const rmMeta = trialJobDetail.rmMeta;
            if (rmMeta !== undefined) {
                await this.writeParameterFile(trialJobId, form.hyperParameters, rmMeta);
            }
            else {
                throw new Error(`updateTrialJob failed: ${trialJobId} rmMeta not found`);
            }
        }
        else {
            throw new Error(`updateTrialJob failed: jobType ${form.jobType} not supported.`);
        }
        return trialJobDetail;
    }
    updateGpuReservation() {
        for (const [key, value] of this.trialJobsMap) {
            if (!['WAITING', 'RUNNING'].includes(value.status)) {
                this.gpuScheduler.removeGpuReservation(value.id, value.rmMeta);
            }
        }
        ;
    }
    get isMultiPhaseJobSupported() {
        return true;
    }
    async cancelTrialJob(trialJobId, isEarlyStopped = false) {
        const deferred = new ts_deferred_1.Deferred();
        const trialJob = this.trialJobsMap.get(trialJobId);
        if (!trialJob) {
            deferred.reject();
            throw new Error(`trial job id ${trialJobId} not found`);
        }
        const index = this.jobQueue.indexOf(trialJobId);
        if (index >= 0) {
            this.jobQueue.splice(index, 1);
        }
        if (trialJob.rmMeta !== undefined) {
            const sshClient = this.trialSSHClientMap.get(trialJob.id);
            if (!sshClient) {
                deferred.reject();
                throw new Error(`Invalid job id ${trialJobId}, cannot find ssh client`);
            }
            const jobpidPath = this.getJobPidPath(trialJob.id);
            try {
                trialJob.isEarlyStopped = isEarlyStopped;
                await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`pkill -P \`cat ${jobpidPath}\``, sshClient);
                this.releaseTrialSSHClient(trialJob);
            }
            catch (error) {
                this.log.error(`remoteTrainingService.cancelTrialJob: ${error.message}`);
            }
        }
        else {
            assert(isEarlyStopped === false, 'isEarlyStopped is not supposed to be true here.');
            trialJob.status = utils_1.getJobCancelStatus(isEarlyStopped);
        }
    }
    async setClusterMetadata(key, value) {
        switch (key) {
            case trialConfigMetadataKey_1.TrialConfigMetadataKey.NNI_MANAGER_IP:
                this.nniManagerIpConfig = JSON.parse(value);
                break;
            case trialConfigMetadataKey_1.TrialConfigMetadataKey.MACHINE_LIST:
                await this.setupConnections(value);
                await cpp.exec(`rm -rf ${this.getLocalGpuMetricCollectorDir()}`);
                break;
            case trialConfigMetadataKey_1.TrialConfigMetadataKey.TRIAL_CONFIG:
                const remoteMachineTrailConfig = JSON.parse(value);
                if (!remoteMachineTrailConfig) {
                    throw new Error('trial config parsed failed');
                }
                if (!fs.lstatSync(remoteMachineTrailConfig.codeDir).isDirectory()) {
                    throw new Error(`codeDir ${remoteMachineTrailConfig.codeDir} is not a directory`);
                }
                try {
                    await util_1.validateCodeDir(remoteMachineTrailConfig.codeDir);
                }
                catch (error) {
                    this.log.error(error);
                    return Promise.reject(new Error(error));
                }
                this.trialConfig = remoteMachineTrailConfig;
                break;
            case trialConfigMetadataKey_1.TrialConfigMetadataKey.MULTI_PHASE:
                this.isMultiPhase = (value === 'true' || value === 'True');
                break;
            case trialConfigMetadataKey_1.TrialConfigMetadataKey.VERSION_CHECK:
                this.versionCheck = (value === 'true' || value === 'True');
                break;
            case trialConfigMetadataKey_1.TrialConfigMetadataKey.LOG_COLLECTION:
                this.logCollection = value;
                break;
            default:
                throw new Error(`Uknown key: ${key}`);
        }
    }
    getClusterMetadata(key) {
        const deferred = new ts_deferred_1.Deferred();
        return deferred.promise;
    }
    async cleanUp() {
        this.log.info('Stopping remote machine training service...');
        this.stopping = true;
        await Promise.race([utils_1.delay(10000), this.cleanupConnections()]);
    }
    async cleanupConnections() {
        try {
            for (const [rmMeta, sshClientManager] of this.machineSSHClientMap.entries()) {
                let jobpidPath = path.join(this.getRemoteScriptsPath(rmMeta.username), 'pid');
                let client = sshClientManager.getFirstSSHClient();
                if (client) {
                    await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`pkill -P \`cat ${jobpidPath}\``, client);
                    await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`rm -rf ${this.getRemoteScriptsPath(rmMeta.username)}`, client);
                }
                sshClientManager.closeAllSSHClient();
            }
        }
        catch (error) {
            this.log.error(`Cleanup connection exception, error is ${error.message}`);
        }
        return Promise.resolve();
    }
    getLocalGpuMetricCollectorDir() {
        let userName = path.basename(os.homedir());
        return `${os.tmpdir()}/${userName}/nni/scripts/`;
    }
    async generateGpuMetricsCollectorScript(userName) {
        let gpuMetricCollectorScriptFolder = this.getLocalGpuMetricCollectorDir();
        await cpp.exec(`mkdir -p ${path.join(gpuMetricCollectorScriptFolder, userName)}`);
        let gpuMetricsCollectorScriptPath = path.join(gpuMetricCollectorScriptFolder, userName, 'gpu_metrics_collector.sh');
        const remoteGPUScriptsDir = this.getRemoteScriptsPath(userName);
        const gpuMetricsCollectorScriptContent = typescript_string_operations_1.String.Format(gpuData_1.GPU_INFO_COLLECTOR_FORMAT_LINUX, remoteGPUScriptsDir, path.join(remoteGPUScriptsDir, 'pid'));
        await fs.promises.writeFile(gpuMetricsCollectorScriptPath, gpuMetricsCollectorScriptContent, { encoding: 'utf8' });
    }
    async setupConnections(machineList) {
        this.log.debug(`Connecting to remote machines: ${machineList}`);
        const deferred = new ts_deferred_1.Deferred();
        const rmMetaList = JSON.parse(machineList);
        let connectedRMNum = 0;
        rmMetaList.forEach(async (rmMeta) => {
            let sshClientManager = new remoteMachineData_1.SSHClientManager([], this.MAX_TRIAL_NUMBER_PER_SSHCONNECTION, rmMeta);
            let sshClient = await sshClientManager.getAvailableSSHClient();
            this.machineSSHClientMap.set(rmMeta, sshClientManager);
            await this.initRemoteMachineOnConnected(rmMeta, sshClient);
            if (++connectedRMNum === rmMetaList.length) {
                deferred.resolve();
            }
        });
        return deferred.promise;
    }
    async initRemoteMachineOnConnected(rmMeta, conn) {
        await this.generateGpuMetricsCollectorScript(rmMeta.username);
        const nniRootDir = `${os.tmpdir()}/nni`;
        await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`mkdir -p ${this.remoteExpRootDir}`, conn);
        const localGpuScriptCollectorDir = this.getLocalGpuMetricCollectorDir();
        const remoteGpuScriptCollectorDir = this.getRemoteScriptsPath(rmMeta.username);
        await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`mkdir -p ${remoteGpuScriptCollectorDir}`, conn);
        await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`chmod 777 ${nniRootDir} ${nniRootDir}/* ${nniRootDir}/scripts/*`, conn);
        await sshClientUtility_1.SSHClientUtility.copyFileToRemote(path.join(localGpuScriptCollectorDir, rmMeta.username, 'gpu_metrics_collector.sh'), path.join(remoteGpuScriptCollectorDir, 'gpu_metrics_collector.sh'), conn);
        sshClientUtility_1.SSHClientUtility.remoteExeCommand(`bash ${path.join(remoteGpuScriptCollectorDir, 'gpu_metrics_collector.sh')}`, conn);
        this.timer.subscribe(async (tick) => {
            const cmdresult = await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`tail -n 1 ${path.join(remoteGpuScriptCollectorDir, 'gpu_metrics')}`, conn);
            if (cmdresult && cmdresult.stdout) {
                rmMeta.gpuSummary = JSON.parse(cmdresult.stdout);
            }
        });
    }
    async prepareTrialJob(trialJobId) {
        const deferred = new ts_deferred_1.Deferred();
        if (!this.trialConfig) {
            throw new Error('trial config is not initialized');
        }
        const trialJobDetail = this.trialJobsMap.get(trialJobId);
        if (trialJobDetail === undefined) {
            throw new errors_1.NNIError(errors_1.NNIErrorNames.INVALID_JOB_DETAIL, `Invalid job detail information for trial job ${trialJobId}`);
        }
        if (trialJobDetail.status !== 'WAITING') {
            deferred.resolve(true);
            return deferred.promise;
        }
        const rmScheduleResult = this.gpuScheduler.scheduleMachine(this.trialConfig.gpuNum, trialJobId);
        if (rmScheduleResult.resultType === remoteMachineData_1.ScheduleResultType.REQUIRE_EXCEED_TOTAL) {
            const errorMessage = `Required GPU number ${this.trialConfig.gpuNum} is too large, no machine can meet`;
            this.log.error(errorMessage);
            deferred.reject();
            throw new errors_1.NNIError(errors_1.NNIErrorNames.RESOURCE_NOT_AVAILABLE, errorMessage);
        }
        else if (rmScheduleResult.resultType === remoteMachineData_1.ScheduleResultType.SUCCEED
            && rmScheduleResult.scheduleInfo !== undefined) {
            const rmScheduleInfo = rmScheduleResult.scheduleInfo;
            const trialWorkingFolder = path.join(this.remoteExpRootDir, 'trials', trialJobId);
            trialJobDetail.rmMeta = rmScheduleInfo.rmMeta;
            await this.allocateSSHClientForTrial(trialJobDetail);
            await this.launchTrialOnScheduledMachine(trialJobId, trialWorkingFolder, trialJobDetail.form, rmScheduleInfo);
            trialJobDetail.status = 'RUNNING';
            trialJobDetail.url = `file://${rmScheduleInfo.rmMeta.ip}:${trialWorkingFolder}`;
            trialJobDetail.startTime = Date.now();
            deferred.resolve(true);
        }
        else if (rmScheduleResult.resultType === remoteMachineData_1.ScheduleResultType.TMP_NO_AVAILABLE_GPU) {
            this.log.info(`Right now no available GPU can be allocated for trial ${trialJobId}, will try to schedule later`);
            deferred.resolve(false);
        }
        else {
            deferred.reject(`Invalid schedule resutl type: ${rmScheduleResult.resultType}`);
        }
        return deferred.promise;
    }
    async launchTrialOnScheduledMachine(trialJobId, trialWorkingFolder, form, rmScheduleInfo) {
        if (!this.trialConfig) {
            throw new Error('trial config is not initialized');
        }
        const cuda_visible_device = rmScheduleInfo.cuda_visible_device;
        const sshClient = this.trialSSHClientMap.get(trialJobId);
        if (sshClient === undefined) {
            assert(false, 'sshClient is undefined.');
            return;
        }
        const trialJobDetail = this.trialJobsMap.get(trialJobId);
        if (trialJobDetail === undefined) {
            throw new Error(`Can not get trial job detail for job: ${trialJobId}`);
        }
        const trialLocalTempFolder = path.join(this.expRootDir, 'trials-local', trialJobId);
        await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`mkdir -p ${trialWorkingFolder}`, sshClient);
        await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`mkdir -p ${path.join(trialWorkingFolder, '.nni')}`, sshClient);
        let command;
        if (typeof cuda_visible_device === 'string' && cuda_visible_device.length > 0) {
            command = `CUDA_VISIBLE_DEVICES=${cuda_visible_device} ${this.trialConfig.command}`;
        }
        else {
            command = `CUDA_VISIBLE_DEVICES=" " ${this.trialConfig.command}`;
        }
        const nniManagerIp = this.nniManagerIpConfig ? this.nniManagerIpConfig.nniManagerIp : utils_1.getIPV4Address();
        if (!this.remoteRestServerPort) {
            const restServer = component.get(remoteMachineJobRestServer_1.RemoteMachineJobRestServer);
            this.remoteRestServerPort = restServer.clusterRestServerPort;
        }
        const version = this.versionCheck ? await utils_2.getVersion() : '';
        const runScriptTrialContent = typescript_string_operations_1.String.Format(remoteMachineData_1.REMOTEMACHINE_TRIAL_COMMAND_FORMAT, trialWorkingFolder, trialWorkingFolder, trialJobId, experimentStartupInfo_1.getExperimentId(), trialJobDetail.sequenceId.toString(), this.isMultiPhase, path.join(trialWorkingFolder, '.nni', 'jobpid'), command, nniManagerIp, this.remoteRestServerPort, version, this.logCollection, path.join(trialWorkingFolder, '.nni', 'code'));
        await cpp.exec(`mkdir -p ${path.join(trialLocalTempFolder, '.nni')}`);
        await cpp.exec(`cp -r ${this.trialConfig.codeDir}/* ${trialLocalTempFolder}`);
        const installScriptContent = containerJobData_1.CONTAINER_INSTALL_NNI_SHELL_FORMAT;
        await fs.promises.writeFile(path.join(trialLocalTempFolder, 'install_nni.sh'), installScriptContent, { encoding: 'utf8' });
        await fs.promises.writeFile(path.join(trialLocalTempFolder, 'run.sh'), runScriptTrialContent, { encoding: 'utf8' });
        await this.writeParameterFile(trialJobId, form.hyperParameters, rmScheduleInfo.rmMeta);
        await sshClientUtility_1.SSHClientUtility.copyDirectoryToRemote(trialLocalTempFolder, trialWorkingFolder, sshClient, this.remoteOS);
        sshClientUtility_1.SSHClientUtility.remoteExeCommand(`bash ${path.join(trialWorkingFolder, 'run.sh')}`, sshClient);
    }
    async runHostJob(form) {
        const rmMeta = this.getRmMetaByHost(form.host);
        const sshClientManager = this.machineSSHClientMap.get(rmMeta);
        if (sshClientManager === undefined) {
            throw new Error('sshClient not found.');
        }
        let sshClient = sshClientManager.getFirstSSHClient();
        const jobId = utils_1.uniqueString(5);
        const localDir = path.join(this.expRootDir, 'hostjobs-local', jobId);
        const remoteDir = this.getHostJobRemoteDir(jobId);
        await cpp.exec(`mkdir -p ${localDir}`);
        await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`mkdir -p ${remoteDir}`, sshClient);
        const runScriptContent = typescript_string_operations_1.String.Format(remoteMachineData_1.HOST_JOB_SHELL_FORMAT, remoteDir, path.join(remoteDir, 'jobpid'), form.cmd, path.join(remoteDir, 'code'));
        await fs.promises.writeFile(path.join(localDir, 'run.sh'), runScriptContent, { encoding: 'utf8' });
        await sshClientUtility_1.SSHClientUtility.copyFileToRemote(path.join(localDir, 'run.sh'), path.join(remoteDir, 'run.sh'), sshClient);
        sshClientUtility_1.SSHClientUtility.remoteExeCommand(`bash ${path.join(remoteDir, 'run.sh')}`, sshClient);
        const jobDetail = new remoteMachineData_1.RemoteMachineTrialJobDetail(jobId, 'RUNNING', Date.now(), remoteDir, form, this.generateSequenceId());
        jobDetail.rmMeta = rmMeta;
        jobDetail.startTime = Date.now();
        this.trialJobsMap.set(jobId, jobDetail);
        this.log.debug(`runHostJob: return: ${JSON.stringify(jobDetail)} `);
        return jobDetail;
    }
    getRmMetaByHost(host) {
        for (const [rmMeta, client] of this.machineSSHClientMap.entries()) {
            if (rmMeta.ip === host) {
                return rmMeta;
            }
        }
        throw new Error(`Host not found: ${host}`);
    }
    async updateTrialJobStatus(trialJob, sshClient) {
        const deferred = new ts_deferred_1.Deferred();
        const jobpidPath = this.getJobPidPath(trialJob.id);
        const trialReturnCodeFilePath = path.join(this.remoteExpRootDir, 'trials', trialJob.id, '.nni', 'code');
        try {
            const killResult = (await sshClientUtility_1.SSHClientUtility.remoteExeCommand(`kill -0 \`cat ${jobpidPath}\``, sshClient)).exitCode;
            if (killResult !== 0) {
                const trailReturnCode = await sshClientUtility_1.SSHClientUtility.getRemoteFileContent(trialReturnCodeFilePath, sshClient);
                this.log.debug(`trailjob ${trialJob.id} return code: ${trailReturnCode}`);
                const match = trailReturnCode.trim().match(/^(\d+)\s+(\d+)$/);
                if (match) {
                    const { 1: code, 2: timestamp } = match;
                    if (parseInt(code, 10) === 0) {
                        trialJob.status = 'SUCCEEDED';
                    }
                    else {
                        if (trialJob.isEarlyStopped === undefined) {
                            trialJob.status = 'FAILED';
                        }
                        else {
                            trialJob.status = utils_1.getJobCancelStatus(trialJob.isEarlyStopped);
                        }
                    }
                    trialJob.endTime = parseInt(timestamp, 10);
                    this.releaseTrialSSHClient(trialJob);
                }
                this.log.debug(`trailJob status update: ${trialJob.id}, ${trialJob.status}`);
            }
            deferred.resolve(trialJob);
        }
        catch (error) {
            this.log.error(`Update job status exception, error is ${error.message}`);
            if (error instanceof errors_1.NNIError && error.name === errors_1.NNIErrorNames.NOT_FOUND) {
                deferred.resolve(trialJob);
            }
            else {
                trialJob.status = 'UNKNOWN';
                deferred.resolve(trialJob);
            }
        }
        return deferred.promise;
    }
    getRemoteScriptsPath(userName) {
        return path.join(utils_1.getRemoteTmpDir(this.remoteOS), userName, 'nni', 'scripts');
    }
    getHostJobRemoteDir(jobId) {
        return path.join(this.remoteExpRootDir, 'hostjobs', jobId);
    }
    getRemoteExperimentRootDir() {
        return path.join(utils_1.getRemoteTmpDir(this.remoteOS), 'nni', 'experiments', experimentStartupInfo_1.getExperimentId());
    }
    get MetricsEmitter() {
        return this.metricsEmitter;
    }
    getJobPidPath(jobId) {
        const trialJobDetail = this.trialJobsMap.get(jobId);
        if (trialJobDetail === undefined) {
            throw new errors_1.NNIError(errors_1.NNIErrorNames.INVALID_JOB_DETAIL, `Invalid job detail information for trial job ${jobId}`);
        }
        let jobpidPath;
        if (trialJobDetail.form.jobType === 'TRIAL') {
            jobpidPath = path.join(trialJobDetail.workingDirectory, '.nni', 'jobpid');
        }
        else if (trialJobDetail.form.jobType === 'HOST') {
            jobpidPath = path.join(this.getHostJobRemoteDir(jobId), 'jobpid');
        }
        else {
            throw new Error(`Job type not supported: ${trialJobDetail.form.jobType}`);
        }
        return jobpidPath;
    }
    async writeParameterFile(trialJobId, hyperParameters, rmMeta) {
        const sshClient = this.trialSSHClientMap.get(trialJobId);
        if (sshClient === undefined) {
            throw new Error('sshClient is undefined.');
        }
        const trialWorkingFolder = path.join(this.remoteExpRootDir, 'trials', trialJobId);
        const trialLocalTempFolder = path.join(this.expRootDir, 'trials-local', trialJobId);
        const fileName = utils_1.generateParamFileName(hyperParameters);
        const localFilepath = path.join(trialLocalTempFolder, fileName);
        await fs.promises.writeFile(localFilepath, hyperParameters.value, { encoding: 'utf8' });
        await sshClientUtility_1.SSHClientUtility.copyFileToRemote(localFilepath, path.join(trialWorkingFolder, fileName), sshClient);
    }
    generateSequenceId() {
        if (this.trialSequenceId === -1) {
            this.trialSequenceId = experimentStartupInfo_1.getInitTrialSequenceId();
        }
        return this.trialSequenceId++;
    }
};
RemoteMachineTrainingService = __decorate([
    component.Singleton,
    __param(0, component.Inject),
    __metadata("design:paramtypes", [observableTimer_1.ObservableTimer])
], RemoteMachineTrainingService);
exports.RemoteMachineTrainingService = RemoteMachineTrainingService;
